#include <stdio.h>
#include <stdlib.h>

/*

diskcopy2raw - Copyright 2004 by Michael Kohn
Webpage: http://www.mikekohn.net/
Email: mike@mikekohn.net

*/


int read_long_b(FILE *in)
{
int c;

  c=getc(in);
  c=(c<<8)+getc(in);
  c=(c<<8)+getc(in);
  c=(c<<8)+getc(in);

  return c;
}

int read_word_b(FILE *in)
{
int c;

  c=getc(in);
  c=(c<<8)+getc(in);

  return c;
}

int read_chars(FILE *in, char *s, int count)
{
int t;

  for (t=0; t<count; t++)
  {
    s[t]=getc(in);
  }

  s[t]=0;

  return 0;
}

unsigned int calc_checksum(FILE *in, int dataSize)
{
long marker;
unsigned int checksum;
int t,r;

  marker=ftell(in);

  checksum=0;

  for (t=0; t<dataSize; t=t+2)
  {
    r=(getc(in)<<8)+getc(in);
    // r=getc(in)+(getc(in)<<8);
    checksum=checksum+r;

    if ((checksum&1)==1)
    { checksum=(checksum>>1)|0x80000000; }
      else
    { checksum=checksum>>1; }
  }

  fseek(in,marker,SEEK_SET);

  return checksum;
}

int main(int argc, char *argv[])
{
FILE *in,*out;
char diskName[65];
int dataSize;
int tagSize;
unsigned int dataChecksum;
unsigned int tagChecksum;
unsigned char diskFormat;
unsigned char formatByte;
int private_word;
int ch,t;
int uhoh_flag;
unsigned int calced_checksum;

  printf("\n\ndiskcopy2raw - Copyright 2004 by Michael Kohn\n");
  printf("For more information: http://www.mikekohn.net/\n\n");

  if (argc!=3)
  {
    printf("Usage: diskcopy2raw <input filename> <output filename>\n");
    exit(0);
  }

  in=fopen(argv[1],"rb");
  if (in==0)
  {
    printf("Could not open file for reading: %s\n",argv[1]);
    exit(1);
  }

  out=fopen(argv[2],"wb");
  if (out==0)
  {
    printf("Could not open file for writing: %s\n",argv[2]);
    fclose(in);
    exit(1);
  }

  uhoh_flag=0;

  read_chars(in,diskName,64);
  dataSize=read_long_b(in);
  tagSize=read_long_b(in);
  dataChecksum=read_long_b(in);
  tagChecksum=read_long_b(in);
  diskFormat=getc(in);
  formatByte=getc(in);
  private_word=read_word_b(in);

  printf("FILE HEADER\n");
  printf("------------------------------\n");
  printf("    diskName: %s\n",diskName);
  printf("    dataSize: %d\n",dataSize);
  printf("     tagSize: %d\n",tagSize);
  printf("dataChecksum: %u\n",dataChecksum);
  printf(" tagChecksum: %d\n",tagChecksum);
  printf("  diskFormat: %d ",diskFormat);
  if (diskFormat==0) printf("400K\n");
    else
  if (diskFormat==1) printf("800K\n");
    else
  if (diskFormat==2) printf("720K\n");
    else
  if (diskFormat==3) printf("1440K\n");
    else
  { printf("Unknown Format\n"); }

  printf("  formatByte: %d ",formatByte);
  if (formatByte==0x12) printf("400K\n");
    else
  if (formatByte==0x22) printf(">400K\n");
    else
  if (formatByte==0x24) printf("800K Apple II disk\n");
    else
  { printf("Unknown Format\n"); }

  printf("     private: %d\n",private_word);
  printf("\n");

  if (private_word!=0x100 || uhoh_flag!=0)
  {
    printf("This image isn't in diskcopy format.  It's probably compressed.\n");

    printf("Try to decompress it first.\n");
  }
    else
  {
    calced_checksum=calc_checksum(in,dataSize);

    if (calced_checksum==dataChecksum)
    { printf("Checksum OK\n"); }
      else
    { printf("Checksum failure!!\n"); }

    for (t=0; t<dataSize; t++)
    {
      ch=getc(in);
      putc(ch,out);
    }
  }


  fclose(out);
  fclose(in);

  return 0;
}



